@@ -54,6 +54,7 @@ import java.util.EnumSet;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Set;
+import java.util.logging.Logger;
 import com.openexchange.client.onboarding.AvailabilityResult;
 import com.openexchange.client.onboarding.BuiltInProvider;
 import com.openexchange.client.onboarding.Device;
@@ -65,6 +66,7 @@ import com.openexchange.client.onboarding.OnboardingUtility;
 import com.openexchange.client.onboarding.Result;
 import com.openexchange.client.onboarding.ResultReply;
 import com.openexchange.client.onboarding.Scenario;
+import com.openexchange.client.onboarding.mail.custom.CustomLoginSource;
 import com.openexchange.client.onboarding.plist.OnboardingPlistProvider;
 import com.openexchange.client.onboarding.plist.PlistResult;
 import com.openexchange.exception.OXException;
@@ -73,6 +75,7 @@ import com.openexchange.groupware.userconfiguration.Permission;
 import com.openexchange.java.Strings;
 import com.openexchange.mail.api.MailConfig;
 import com.openexchange.mail.service.MailService;
+import com.openexchange.mail.transport.config.TransportAuthSupportAware;
 import com.openexchange.mail.transport.config.TransportConfig;
 import com.openexchange.mail.usersetting.UserSettingMail;
 import com.openexchange.mail.usersetting.UserSettingMailStorage;
@@ -82,6 +85,7 @@ import com.openexchange.plist.PListDict;
 import com.openexchange.server.ServiceExceptionCode;
 import com.openexchange.server.ServiceLookup;
 import com.openexchange.session.Session;
+import com.openexchange.sessiond.SessionMatcher;
 import com.openexchange.sessiond.SessiondService;
 import com.openexchange.user.UserService;
 
@@ -99,6 +103,8 @@ public class MailOnboardingProvider implements OnboardingPlistProvider {
     private final Set<Device> supportedDevices;
     private final Set<OnboardingType> supportedTypes;
 
+    private static final Logger LOG = Logger.getLogger(MailOnboardingProvider.class.getName());
+
     /**
      * Initializes a new {@link MailOnboardingProvider}.
      */
@@ -206,7 +212,21 @@ public class MailOnboardingProvider implements OnboardingPlistProvider {
             if (null == imapSecure) {
                 imapSecure = Boolean.valueOf(imapConfig.isSecure());
             }
-            String imapLogin = imapConfig.getLogin();
+
+            Boolean customSource = OnboardingUtility.getBoolFromProperty("com.openexchange.client.onboarding.mail.imap.login.customsource", false, session);
+            String imapLogin;
+
+            if (customSource) {
+                CustomLoginSource customLoginSource = services.getService(CustomLoginSource.class);
+                if(customLoginSource==null){
+                    LOG.warning("Unable to find any CustomLoginSource services! Falling back to imap config.");
+                    imapLogin = imapConfig.getLogin();
+                } else {
+                    imapLogin = customLoginSource.getImapLogin(session);
+                }
+            } else {
+                imapLogin = imapConfig.getLogin();
+            }
             String imapPassword = imapConfig.getPassword();
             imapConfiguration = new Configuration(imapServer, imapPort.intValue(), imapSecure.booleanValue(), imapLogin, imapPassword);
         }
@@ -228,8 +248,27 @@ public class MailOnboardingProvider implements OnboardingPlistProvider {
             if (null == smtpSecure) {
                 smtpSecure = Boolean.valueOf(smtpConfig.isSecure());
             }
-            String smtpLogin = smtpConfig.getLogin();
             String smtpPassword = smtpConfig.getPassword();
+
+            Boolean customSource = OnboardingUtility.getBoolFromProperty("com.openexchange.client.onboarding.mail.smtp.login.customsource", false, session);
+            String smtpLogin;
+
+            if (customSource) {
+                CustomLoginSource customLoginSource = services.getService(CustomLoginSource.class);
+                if(customLoginSource==null){
+                    LOG.warning("Unable to find any CustomLoginSource services! Falling back to smtp config.");
+                    smtpLogin = smtpConfig.getLogin();
+                }else {
+                    smtpLogin = customLoginSource.getSmtpLogin(session);
+                }
+            } else {
+                smtpLogin = smtpConfig.getLogin();
+            }
+
+            if ((smtpConfig instanceof TransportAuthSupportAware) && (false == ((TransportAuthSupportAware) smtpConfig).isAuthSupported())) {
+                smtpLogin = null;
+                smtpPassword = null;
+            }
             smtpConfiguration = new Configuration(smtpServer, smtpPort.intValue(), smtpSecure.booleanValue(), smtpLogin, smtpPassword);
         }
 
@@ -248,6 +287,29 @@ public class MailOnboardingProvider implements OnboardingPlistProvider {
             throw OXException.general("No active session for user " + userId + " in context " + contextId);
         }
 
+        if (session.getPassword() == null) {
+            // Try to find a session with password!=null
+            SessionMatcher matcher = new SessionMatcher() {
+
+                @Override
+                public Set<Flag> flags() {
+                    return SessionMatcher.NO_FLAGS;
+                }
+
+                @Override
+                public boolean accepts(Session session) {
+                    if (session.getPassword() != null) {
+                        return true;
+                    }
+                    return false;
+                }
+            };
+            session = sessiondService.findFirstMatchingSessionForUser(userId, contextId, matcher);
+            if (null == session) {
+                throw OXException.general("No active session for user " + userId + " in context " + contextId);
+            }
+        }
+
         return getEffectiveConfigurations(session);
     }
 
